// ==========================================================================
// File: OP-RUMR.MPS
// Desc: Optimized random rumor selector for multinode Mystic BBS v1.12a46
// Author: opicron
// Original script: g00r00
// ==========================================================================
//
// INSTALLATION:
//
//   1) Copy OP-RUMR.MPS into your script path
//   2) Copy RUMOR.DAT into /op-rumr/ folder in script path
//   3) compile OP-RUMR.MPS with MPLC or MIDE
//
// SETUP:
//
//   1) Each menu/program you want to display an rumor on needs an added
//      action list command, as follows:
//
//      mystic -cfg -> Menu Editor -> your menu entry action list
//
//         HotKey: EVERY
//        Command: GX (Execute MPL)
//           Data: op-rumr load
//
//   2) When "rumors load" is called upon, it checks the rumor file selects
//      an rumor (if time for node has passed) and stores it in an tmp file.
//      The rumor is added to the |&1 MCI code. You will need to add the MCI
//      code to our menu ANSI or prompt.
//
// CUSTOMIZATION:
//
//   If you need more rumors add them to the rumor.dat file! Note that the
//   maximum is set at 1000 lines. Could easily be more though.
//
//   If you feel the need to increase/descrease the rumor timeout see the
//   constant ReloadTimer below.
//
//  TEMP FILES:
//
//    Two temp files are created in the /op-rumr/ folder per node.
//
//    op-rumr.s[node]   saved rumor to retrieve
//    op-rumr.t[node]   time to check reload timer
//
// ==========================================================================

{$syntax iplc}

Const
  RumorFilePath   = "op-rumr/";         // PROMPT SUB FOLDER
  ReloadTimer = 10
  
proc getrumor():String {
  @ string data
  @ integer datasize, count
  @ file f

  fassign (f, justpath(progname) + RumorFilePath + "rumors.dat", 2)
  freset (f)

  if ioresult != 0 exit

  while !feof(f) && datasize < 1000 { // increased to 1000
    datasize = datasize + 1
    freadln(f, data)
  }

  count    = random(datasize) + 1
  datasize = 0

  freset(f)
  while datasize != count {
    datasize = datasize + 1
    freadln(f, data)
  }
  fclose(f) // rumors.dat
  //data now has random rumour!

  getrumor = data
}

proc rumorload {

  @ string data, time, olddata
  @ file f, ftmp

  //open file to write new rumour if time passed or time file does not exist
  fassign (ftmp, justpath(progname) + RumorFilePath + "op-rumr.s" + int2str(nodenum), 2)

  //open time check file
  fassign (f, justpath(progname) + RumorFilePath + "op-rumr.t" + int2str(nodenum), 2)
  freset (f)

  // set first time if file not exists
  if ioresult != 0 {
    frewrite(f);
    fwriteln(f, int2str(datetime));

    //get rumor
    data = getrumor()
    
    //write tmp rumour
    frewrite(ftmp)
    fwriteln(ftmp, data);

  // check time if file exists
  } else {

    // read time
    freadln(f, time)

    // compare with current time
    if str2int(time)+ReloadTimer < datetime {
      frewrite(f);
      fwriteln(f, int2str(datetime));

      //get rumor
      data = getrumor()
      
      //write tmp rumour
      frewrite(ftmp)
      fwriteln(ftmp, data);

    }
  }
  fclose(f) //rumors.t

  freset(ftmp) //opened while back
  freadln(ftmp, olddata)
  fclose(ftmp) //rumors.s

  // export rumour
  setpromptinfo(1, olddata) // i wish we could use A-Z prompts! :(
}

{
  @ string options = upper(progparams);

  if pos("LOAD", options) > 0
    rumorload()
  else
    writeln("OP-RUMR: missing argument..|CR|PA")
}
